// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "bhv_cross_situation_move.h"


#include <rcsc/action/basic_actions.h>
#include <rcsc/action/body_go_to_point.h>
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/neck_turn_to_ball.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>
#include <rcsc/player/intercept_table.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

/*-------------------------------------------------------------------*/
/*!
  execute action
*/
bool
Bhv_CrossSituationMove::execute( rcsc::PlayerAgent * agent )
{
    const rcsc::WorldModel & wm = agent->world();

    // go to shoot position
    const double shoot_y = rcsc::ServerParam::i().goalHalfWidth() ;
    
    if ( wm.self().pos().y < 0.0 )  shoot_y * ( -1.0 ) ;
    
    const rcsc::Vector2D shoot_pos( 40.0 , shoot_y ) ;
    const double dash_power = rcsc::ServerParam::i().maxPower() ;

    double dist_thr = ( shoot_pos - wm.self().pos() ).length() * 0.3 ;
    
    if ( dist_thr < 3.0 ) dist_thr = 3.0 ;

    agent->debugClient().addMessage( " Go to Shoot Pos" );
    agent->debugClient().setTarget( shoot_pos );

    rcsc::dlog.addText( rcsc::Logger::TEAM,
                        "%s:%d: Bhv_CrossSituationMove ( %.1f, %.1f )"
                        ,__FILE__, __LINE__,
			shoot_pos.x, shoot_pos.y );


    if ( ! rcsc::Body_GoToPoint( shoot_pos, dist_thr, dash_power
                                 ).execute( agent ) )
    {
      rcsc::Body_TurnToBall().execute( agent );
    }
    
    if ( wm.existKickableOpponent()
	 && wm.ball().distFromSelf() < 18.0 )
      {
	agent->setNeckAction( new rcsc::Neck_TurnToBall() );
      }
    else
      {
	agent->setNeckAction( new rcsc::Neck_TurnToBallOrScan() );
      }
    
    return true;
}

