// -*-c++-*-

/*
 *Copyright:

 Copyright (C) Hidehisa AKIYAMA

 This code is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This code is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this code; see the file COPYING.  If not, write to
 the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

 *EndCopyright:
 */

/////////////////////////////////////////////////////////////////////

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "bhv_defense_move.h"


#include <rcsc/action/basic_actions.h>
#include <rcsc/action/body_go_to_point.h>
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/neck_turn_to_ball.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>
#include <rcsc/player/intercept_table.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

/*-------------------------------------------------------------------*/
/*!
  execute action
*/
bool
Bhv_DefenseMove::execute( rcsc::PlayerAgent * agent )
{
    rcsc::dlog.addText( rcsc::Logger::TEAM,
                        "%s:%d: Bhv_DefenseMove"
                        ,__FILE__, __LINE__ );

    const rcsc::WorldModel & wm = agent->world();
    const rcsc::PlayerObject * opp
      = wm.interceptTable()->fastestOpponent() ;
    
    // check ball owner

    int self_min = wm.interceptTable()->selfReachCycle();
    int mate_min = wm.interceptTable()->teammateReachCycle();
    int opp_min = wm.interceptTable()->opponentReachCycle();

    if ( opp 
	 && self_min < mate_min 
	 && self_min > opp_min ) {
      rcsc::Vector2D block_point = opp->pos() ;
      if ( wm.self().pos().x > opp->pos().x ) {
	block_point.x -= 5.0 ;
	block_point.y -= 5.0 ;
      } else {
	block_point.x -= 2.0 ;
	block_point.y -= 2.0 ;
      }
      
      if ( rcsc::Body_GoToPoint( block_point,
				 1.0,
				 rcsc::ServerParam::i().maxPower(),
				 5,
				 false,
				 true,
				 40.0 ).execute( agent ) 
	   )
	{
	  rcsc::dlog.addText( rcsc::Logger::TEAM,
			      "%s:%d: Block Dribble %lf"
			      ,__FILE__ ,__LINE__ ,block_point ) ;
	  return true ;
	}
    }

    return true ;
}

